"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tmp_1 = __importDefault(require("tmp"));
const NodeFS_1 = require("./NodeFS");
const path_1 = require("./path");
var path_2 = require("./path");
exports.PortablePath = path_2.PortablePath;
var path_3 = require("./path");
exports.npath = path_3.npath;
exports.ppath = path_3.ppath;
exports.toFilename = path_3.toFilename;
var AliasFS_1 = require("./AliasFS");
exports.AliasFS = AliasFS_1.AliasFS;
var FakeFS_1 = require("./FakeFS");
exports.FakeFS = FakeFS_1.FakeFS;
var CwdFS_1 = require("./CwdFS");
exports.CwdFS = CwdFS_1.CwdFS;
var JailFS_1 = require("./JailFS");
exports.JailFS = JailFS_1.JailFS;
var LazyFS_1 = require("./LazyFS");
exports.LazyFS = LazyFS_1.LazyFS;
var NoFS_1 = require("./NoFS");
exports.NoFS = NoFS_1.NoFS;
var NodeFS_2 = require("./NodeFS");
exports.NodeFS = NodeFS_2.NodeFS;
var PosixFS_1 = require("./PosixFS");
exports.PosixFS = PosixFS_1.PosixFS;
var ProxiedFS_1 = require("./ProxiedFS");
exports.ProxiedFS = ProxiedFS_1.ProxiedFS;
var VirtualFS_1 = require("./VirtualFS");
exports.VirtualFS = VirtualFS_1.VirtualFS;
var ZipFS_1 = require("./ZipFS");
exports.ZipFS = ZipFS_1.ZipFS;
var ZipOpenFS_1 = require("./ZipOpenFS");
exports.ZipOpenFS = ZipOpenFS_1.ZipOpenFS;
function patchFs(patchedFs, fakeFs) {
    const SYNC_IMPLEMENTATIONS = new Set([
        `accessSync`,
        `appendFileSync`,
        `createReadStream`,
        `chmodSync`,
        `closeSync`,
        `copyFileSync`,
        `lstatSync`,
        `mkdirSync`,
        `openSync`,
        `readSync`,
        `readlinkSync`,
        `readFileSync`,
        `readdirSync`,
        `readlinkSync`,
        `realpathSync`,
        `renameSync`,
        `rmdirSync`,
        `statSync`,
        `symlinkSync`,
        `unlinkSync`,
        `utimesSync`,
        `watch`,
        `writeFileSync`,
        `writeSync`,
    ]);
    const ASYNC_IMPLEMENTATIONS = new Set([
        `accessPromise`,
        `appendFilePromise`,
        `chmodPromise`,
        `closePromise`,
        `copyFilePromise`,
        `lstatPromise`,
        `mkdirPromise`,
        `openPromise`,
        `readdirPromise`,
        `realpathPromise`,
        `readFilePromise`,
        `readdirPromise`,
        `readlinkPromise`,
        `renamePromise`,
        `rmdirPromise`,
        `statPromise`,
        `symlinkPromise`,
        `unlinkPromise`,
        `utimesPromise`,
        `writeFilePromise`,
        `writeSync`,
    ]);
    patchedFs.existsSync = (p) => {
        try {
            return fakeFs.existsSync(p);
        }
        catch (error) {
            return false;
        }
    };
    patchedFs.exists = (p, ...args) => {
        const hasCallback = typeof args[args.length - 1] === `function`;
        const callback = hasCallback ? args.pop() : () => { };
        process.nextTick(() => {
            fakeFs.existsPromise(p).then(exists => {
                callback(exists);
            }, () => {
                callback(false);
            });
        });
    };
    patchedFs.read = (p, buffer, ...args) => {
        const hasCallback = typeof args[args.length - 1] === `function`;
        const callback = hasCallback ? args.pop() : () => { };
        process.nextTick(() => {
            fakeFs.readPromise(p, buffer, ...args).then(bytesRead => {
                callback(undefined, bytesRead, buffer);
            }, error => {
                callback(error);
            });
        });
    };
    for (const fnName of ASYNC_IMPLEMENTATIONS) {
        const fakeImpl = fakeFs[fnName].bind(fakeFs);
        const origName = fnName.replace(/Promise$/, ``);
        patchedFs[origName] = (...args) => {
            const hasCallback = typeof args[args.length - 1] === `function`;
            const callback = hasCallback ? args.pop() : () => { };
            process.nextTick(() => {
                fakeImpl(...args).then((result) => {
                    callback(undefined, result);
                }, (error) => {
                    callback(error);
                });
            });
        };
    }
    for (const fnName of SYNC_IMPLEMENTATIONS) {
        const fakeImpl = fakeFs[fnName].bind(fakeFs);
        const origName = fnName;
        patchedFs[origName] = fakeImpl;
    }
    patchedFs.realpathSync.native = patchedFs.realpathSync;
    patchedFs.realpath.native = patchedFs.realpath;
}
exports.patchFs = patchFs;
function extendFs(realFs, fakeFs) {
    const patchedFs = Object.create(realFs);
    patchFs(patchedFs, fakeFs);
    return patchedFs;
}
exports.extendFs = extendFs;
exports.xfs = Object.assign(new NodeFS_1.NodeFS(), {
    mktempSync(cb) {
        const { name, removeCallback } = tmp_1.default.dirSync({ unsafeCleanup: true });
        if (typeof cb === `undefined`) {
            return path_1.npath.toPortablePath(name);
        }
        else {
            try {
                return cb(path_1.npath.toPortablePath(name));
            }
            finally {
                removeCallback();
            }
        }
    },
    mktempPromise(cb) {
        if (typeof cb === `undefined`) {
            return new Promise((resolve, reject) => {
                tmp_1.default.dir({ unsafeCleanup: true }, (err, path) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(path_1.npath.toPortablePath(path));
                    }
                });
            });
        }
        else {
            return new Promise((resolve, reject) => {
                tmp_1.default.dir({ unsafeCleanup: true }, (err, path, cleanup) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        Promise.resolve(path_1.npath.toPortablePath(path)).then(cb).then(result => {
                            cleanup();
                            resolve(result);
                        }, error => {
                            cleanup();
                            reject(error);
                        });
                    }
                });
            });
        }
    },
});
